package com.oracle.coherencebpelintregration.eventlistener;

import com.tangosol.net.CacheFactory;
import com.tangosol.net.NamedCache;
import com.tangosol.util.MapEvent;
import com.tangosol.util.MapListener;

import org.w3c.dom.Element;

import com.collaxa.xml.XMLHelper;


import com.oracle.bpel.client.Locator;
import com.oracle.bpel.client.NormalizedMessage;
import com.oracle.bpel.client.delivery.IDeliveryService;


import java.util.Map;
import java.util.Properties;


/*  This Java class listens to new cache events,
 *  filters events for the SayHelloWorldSynch BPEL process,
 *  calls this process using the BPEL Java API,
 *  and pushes the response back into the cache
 *
 *  @author: ingo.laue@oracle.com, 16-Oct-2007
 */


public class SayHelloWorldCaller implements Runnable {

    Thread myThread;

    Properties props = new java.util.Properties();
    NamedCache nc;


    public SayHelloWorldCaller() {

    }


    public void startListener() {
        nc = CacheFactory.getCache("VirtualCache");


        props.setProperty("orabpel.platform", "ias_10g");
        props.setProperty("java.naming.factory.initial", 
                          "com.evermind.server.rmi.RMIInitialContextFactory");
        props.setProperty("java.naming.provider.url", 
                          "opmn:ormi://ilaue-de:6004:home/orabpel");
        props.setProperty("java.naming.security.principal", "oc4jadmin");
        props.setProperty("java.naming.security.credentials", "welcome1");
        props.setProperty("dedicated.connection", "true");
        props.setProperty("dedicated.rmicontext", "true");


        nc.addMapListener(new MapListener() {

                    public void entryInserted(MapEvent me) {
                        System.out.println("---------------------");
                        System.out.println("New Map event discovered at " + 
                                           new java.util.Date().toString());
                        System.out.println("Key: " + me.getKey());
                        System.out.println("Value: " + me.getNewValue());

                        String key = (String)me.getKey();
                        String value = (String)me.getNewValue();

                        if (key.startsWith("SayHelloProcessRequest")) {
                            callSayHelloBPELProcess(key, value);
                        }

                    }

                    public void entryDeleted(MapEvent me) {
                        System.out.println("---------------------");
                        System.out.println("Map entry deleted at " + 
                                           new java.util.Date().toString());
                        System.out.println("Key: " + me.getKey());
                    }

                    public void entryUpdated(MapEvent me) {
                    }


                });


        if (myThread == null) {
            myThread = new Thread(this);
            myThread.start();
        }


        System.out.println("Initialiation complete. Listener active.");


    }


    private void callSayHelloBPELProcess(String key, String value) {

        String xml = 
            "<SayHelloSynchProcessProcessRequest xmlns=\"http://xmlns.oracle.com/SayHelloSynchProcess\">\n" + 
            "   <input>" + value + "</input>\n" + 
            "</SayHelloSynchProcessProcessRequest>";

        try {
            Locator locator = new Locator("default", "bpel", props);
            IDeliveryService deliveryService = 
                (IDeliveryService)locator.lookupService(IDeliveryService.SERVICE_NAME);


            NormalizedMessage nm = new NormalizedMessage();
            nm.addPart("payload", xml);


            System.out.println(xml);

            NormalizedMessage res = 
                deliveryService.request("SayHelloSynchProcess", "process", nm);

            String response = "";


            Map payload = res.getPayload();

            Element el = (Element)payload.get("payload");
            response=XMLHelper.toXML(el);
            System.out.println(response);
            response=response.substring(response.indexOf("<result>")+"<result>".length(), response.indexOf("</result>"));
            System.out.println(response);
            nc.put("SayHelloProcessResponse" + 
                   key.substring("SayHelloProcessRequest".length(), 
                                 key.length()), response);
            nc.remove(key);

        } catch (Exception e) {
            e.printStackTrace();
        }


    }


    public static void main(String[] args) throws Exception {


        SayHelloWorldCaller sml = new SayHelloWorldCaller();
        sml.startListener();


    }


    public void run() {
        while (true) {
            try {
                myThread.sleep(2000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }
}
